/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import sjima.cgf.layer.CGFBorderTerrainLayer;
import sjima.cgf.layer.CGFCursolLayer;
import sjima.cgf.layer.CGFDialogLayer;
import sjima.cgf.layer.CGFInfoBoardLayer;
import sjima.cgf.layer.CGFTerrainLayer;

public class CGFGameData {
	private final static Color CLEAR_COLOR1 = new Color(0, 0, 0, 0);
	private final static Color CLEAR_COLOR2 = new Color(255, 255, 255, 255);
	public static final int MAX_PLAYER = 9;
	private int maxPlayer = MAX_PLAYER;

	private CGFSize mapSize;
	private CGFRowColsTable cellTable;
	private CGFMain main;
	private BufferedImage boardImage;
	private CGFSize squareSize = new CGFSize(16, 16);
	private LinkedHashMap layerList = new LinkedHashMap();
	private CGFMode currentMode;
	private LinkedHashMap modeList = new LinkedHashMap();
	private CGFLocation cursolLocation;
	private Map terrainMap = new HashMap();
	private CGFTerrain defaultTerrain;
	private Map borderTerrainMap = new HashMap();
	private Map handlerMap = new HashMap();
	private List playerList = new ArrayList();
	private Map playerMap = new HashMap();
	private List colorList = new ArrayList();
	private int ninzu;
	private CGFTurnManager turnManager;
	private CGFPhaseManager phaseManager;

	public CGFGameData(CGFMain p_main, CGFSize p_size) {
		main = p_main;
		setMapSize(p_size);
	}

	public CGFGameData(CGFMain p_main) {
		main = p_main;
	}

	public void clear() {
		setBoardImage(null);
		getLayerList().clear();
		getModeList().clear();
		getTerrainMap().clear();
		getBorderTerrainMap().clear();
		getHandlerMap().clear();
		getPlayerList().clear();
		getPlayerMap().clear();
		getColorList().clear();
	}

	public void buildCellTable() {
		setCellTable(new CGFRowColsTable(mapSize, getMain().getFactory()
				.getCellBuilder()));
	}

	public void buildBoardImage() {
		setBoardImage(CGFUtil.createImage(getBoardImageSize()));
	}

	public void createDefaultTerrain() {
		CGFTerrain terrain = getMain().getFactory().createTerrain("PL",
				"resource/plain.png", "平地");
		setDefaultTerrain(terrain);
	}

	public void buildModes() {
		modeList = new LinkedHashMap();
		CGFMode mode = new CGFMode(main);
		modeList.put(mode.getModeName(), mode);
		mode = new CGFDialogMode(main);
		modeList.put(mode.getModeName(), mode);
	}

	public void buildLayers() {
		layerList = new LinkedHashMap();
		CGFLayer layer = new CGFTerrainLayer(main, getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new CGFBorderTerrainLayer(main, getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new CGFCursolLayer(main, getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new CGFInfoBoardLayer(main, getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new CGFDialogLayer(main, getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
	}

	public CGFSize getBoardImageSize() {
		int BoardImageWidth = mapSize.width * squareSize.width;
		int BoardImageHeight = mapSize.height * squareSize.height;
		CGFSize BoardImageSize = new CGFSize(BoardImageWidth, BoardImageHeight);
		return BoardImageSize;
	}

	public synchronized void repaintBoard() {
		Graphics2D gc = getBoardImage().createGraphics();
		if (getMain().getMainFrame().getOpeningImage() != null) {
			gc.setBackground(CLEAR_COLOR1);
		} else {
			gc.setBackground(CLEAR_COLOR2);
		}
		gc.clearRect(0, 0, getBoardImage().getWidth(), getBoardImage()
				.getHeight());
		for (Iterator ite = getLayerList().values().iterator(); ite.hasNext();) {
			CGFLayer layer = (CGFLayer) ite.next();
			layer.update();
			layer.paint(boardImage);
		}
		main.getMainFrame().setImage(boardImage);
	}

	public void repaintRectangleList(List list) {
		if (getBoardImage() == null) {
			return;
		}
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Rectangle rect = (Rectangle) ite.next();
			if (rect == null) {
				continue;
			}
			repaintRectangle(rect);
			getMain().getMainFrame().updateImage(getBoardImage(), rect);
		}
	}

	public void repaintRectangle(Rectangle rect) {
		rect = rect.intersection(new Rectangle(0, 0,
				getBoardImage().getWidth(), getBoardImage().getHeight()));
		Graphics2D gc = getBoardImage().createGraphics();
		if (getMain().getMainFrame().getOpeningImage() != null) {
			gc.setBackground(CLEAR_COLOR1);
		} else {
			gc.setBackground(CLEAR_COLOR2);
		}
		gc.clearRect(rect.x, rect.y, rect.width, rect.height);
		for (Iterator ite = getLayerList().values().iterator(); ite.hasNext();) {
			CGFLayer layer = (CGFLayer) ite.next();
			layer.paintRectangle(boardImage, rect);
		}
	}

	public CGFCell getCell(CGFLocation loc) {
		return (CGFCell) cellTable.getData(loc);
	}

	public void setTerrain(CGFLocation loc, CGFTerrain terrain) {
		cellTable.setData(loc, terrain);
	}

	public boolean equals(Object obj) {
		return mapSize.equals(obj);
	}

	public int hashCode() {
		return mapSize.hashCode();
	}

	public String toString() {
		return mapSize.toString();
	}

	public CGFSize getMapSize() {
		return mapSize;
	}

	public void setMapSize(CGFSize mapSize) {
		this.mapSize = mapSize;
		setCursolLocation(null);
		createDefaultTerrain();
		buildCellTable();
		buildBoardImage();
		buildLayers();
		buildModes();
	}

	public CGFSize getSquareSize() {
		return squareSize;
	}

	public void setSquareSize(CGFSize squareSize) {
		this.squareSize = squareSize;
	}

	public CGFRowColsTable getCellTable() {
		return cellTable;
	}

	public void setCellTable(CGFRowColsTable terrainTable) {
		this.cellTable = terrainTable;
	}

	public CGFMain getMain() {
		return main;
	}

	public void setMain(CGFMain main) {
		this.main = main;
	}

	public BufferedImage getBoardImage() {
		return boardImage;
	}

	public void setBoardImage(BufferedImage boardImage) {
		this.boardImage = boardImage;
	}

	public Point getLocationPoint(CGFLocation loc) {
		Point pos = new Point(loc.col * squareSize.width, loc.row
				* squareSize.height);

		return pos;
	}

	public CGFLocation getLocationAtPoint(Point pos) {
		int row = pos.y / getSquareSize().height;
		int col = pos.x / getSquareSize().width;
		return new CGFLocation(row, col);
	}

	public CGFMode getCurrentMode() {
		return currentMode;
	}

	public void setCurrentMode(CGFMode currentMode) {
		this.currentMode = currentMode;
	}

	public void setDisplayMode() {
		CGFMode displayMode = (CGFMode) getModeList().get(CGFMode.MODE_NAME);
		setCurrentMode(displayMode);
		repaintBoard();
	}

	public LinkedHashMap getLayerList() {
		return layerList;
	}

	public void setLayerList(LinkedHashMap layerList) {
		this.layerList = layerList;
	}

	public LinkedHashMap getModeList() {
		return modeList;
	}

	public void setModeList(LinkedHashMap modeList) {
		this.modeList = modeList;
	}

	public CGFLocation getCursolLocation() {
		return cursolLocation;
	}

	public void setCursolLocation(CGFLocation cursolLocation) {
		this.cursolLocation = cursolLocation;
	}

	public CGFLayer getLayer(String layerName) {
		return (CGFLayer) getLayerList().get(layerName);
	}

	public Map getTerrainMap() {
		return terrainMap;
	}

	public void setTerrainMap(Map terrainMap) {
		this.terrainMap = terrainMap;
	}

	public CGFTerrain getDefaultTerrain() {
		return defaultTerrain;
	}

	public void setDefaultTerrain(CGFTerrain defaultTerrain) {
		this.defaultTerrain = defaultTerrain;
	}

	public Map getBorderTerrainMap() {
		return borderTerrainMap;
	}

	public void setBorderTerrainMap(Map borderTerrainMap) {
		this.borderTerrainMap = borderTerrainMap;
	}

	public synchronized void changeCurrentMode(String modeName) {
		CGFMode mode = (CGFMode) this.getModeList().get(modeName);
		setCurrentMode(mode);
		if (mode.updateLayers()) {
			this.repaintBoard();
		}
	}

	public Map createHandlerMap() {
		CGFDialogMode mode = (CGFDialogMode) this.getModeList().get(
				CGFDialogMode.MODE_NAME);
		Map map = mode.getHandlerMap();
		setHandlerMap(map);
		return map;
	}

	public Map getHandlerMap() {
		return handlerMap;
	}

	public void setHandlerMap(Map handlerMap) {
		this.handlerMap = handlerMap;
	}

	public List getPlayerList() {
		return playerList;
	}

	public void setPlayerList(List playerList) {
		this.playerList = playerList;
	}

	public void createPlayerMap() {
		setPlayerMap(new HashMap());
		for (Iterator ite = getPlayerList().iterator(); ite.hasNext();) {
			CGFPlayer player = (CGFPlayer) ite.next();
			getPlayerMap().put(player.getNo(), player);
		}
	}

	public CGFPlayer getPlayer(String no) {
		return (CGFPlayer) getPlayerMap().get(no);
	}

	public List getColorList() {
		return colorList;
	}

	public void setColorList(List colorList) {
		this.colorList = colorList;
	}

	public int getNinzu() {
		return ninzu;
	}

	public void setNinzu(int ninzu) {
		this.ninzu = ninzu;
	}

	public int getMaxPlayer() {
		return maxPlayer;
	}

	public void setMaxPlayer(int max) {
		this.maxPlayer = max;
	}

	public CGFTurnManager getTurnManager() {
		return turnManager;
	}

	public void setTurnManager(CGFTurnManager turnManager) {
		this.turnManager = turnManager;
	}

	public CGFPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public void setPhaseManager(CGFPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	private Map getPlayerMap() {
		return playerMap;
	}

	private void setPlayerMap(Map playerMap) {
		this.playerMap = playerMap;
	}

	public void scrollToLocation(CGFLocation loc) {
		String autoScroll = getMain().getSetting().getStringValue("autoscroll");
		if ((autoScroll != null) && (!"1".equals(autoScroll))) {
			return;
		}
		clearInfoBoard();
		getMain().getMainFrame().scrollToPoint(getLocationPoint(loc));
		drawInfoBoard();
	}

	public void drawInfoBoard() {
		CGFInfoBoardLayer layer = (CGFInfoBoardLayer) getLayer(CGFInfoBoardLayer.LAYER_NAME);
		List list = layer.drawBoard();
		this.repaintRectangleList(list);
	}

	public void clearInfoBoard() {
		CGFInfoBoardLayer layer = (CGFInfoBoardLayer) getLayer(CGFInfoBoardLayer.LAYER_NAME);
		List list = layer.clearBoard();
		this.repaintRectangleList(list);
	}

	public synchronized void closeDialog() {
		((CGFDialogMode) (getModeList().get(CGFDialogMode.MODE_NAME)))
				.closeDialog();
		((CGFDialogMode) (getModeList().get(CGFDialogMode.MODE_NAME)))
				.closeMenu();
		if (getPhaseManager() != null) {
			CGFMode defaultMode = getPhaseManager().getCurrentPhase()
					.getDefaultMode();
			setCurrentMode(defaultMode);
		}
	}

	public Rectangle getBoardRectangle() {
		return new Rectangle(0, 0, getBoardImage().getWidth(), getBoardImage()
				.getHeight());
	}
}
