/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

public class CGFMain {
	private CGFMainFrame mainFrame;
	private CGFGameData gameData;
	private CGFFactory factory;
	private CGFDialogDocument dialogDocument;
	private CGFMenuDocument menuDocument;
	private JFileChooser saveFileChooser;
	private JFileChooser soundFileChooser;
	private JFileChooser settingFileChooser;
	private CGFFontManager fontManager;
	private CGFSetting setting;
	private CGFSoundManager soundManager = null;

	public static final String XML_EXT = "xml";

	public static void main(String argv[]) {
		CGFMain main = new CGFMain();
		main.start();
	}

	public CGFMain() {
		createSaveFileChooser();
		createSoundFileChooser();
		createSettingFileChooser();
	}

	public void createSaveFileChooser() {
		setSaveFileChooser(new JFileChooser());
	}

	public void createSoundFileChooser() {
		JFileChooser chooser = new JFileChooser();
		CGFFileFilter filter = new CGFFileFilter();
		filter.add(CGFSoundSetting.SOUND_MP3_EXT);
		filter.add(CGFSoundSetting.SOUND_MP3_EXT.toUpperCase());
		filter.add(CGFSoundSetting.SOUND_MIDI_EXT);
		filter.add(CGFSoundSetting.SOUND_MIDI_EXT.toUpperCase());
		filter.setDescription("MIDI/MP3 Sound File(*.mid,*.mp3)");
		chooser.setFileFilter(filter);
		setSoundFileChooser(chooser);

		if (getSoundManager() == null) {
			return;
		}
		String dir = getSoundSetting().getSoundFileDirectory();
		if (dir != null) {
			// OpenJDKでは時々ここでなぞの例外が起きるのだが(swing内、なんのことだからさっぱり・・・）
			// 仕方がないので、例外発生時は結果を出力して強行
			try {
				getSoundFileChooser().setCurrentDirectory(new File(dir));
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void createSettingFileChooser() {
		JFileChooser chooser = new JFileChooser();
		CGFFileFilter filter = new CGFFileFilter();
		filter.add(XML_EXT);
		filter.addExList("setting.xml");
		filter.addExList("font.xml");
		filter.addExList("Dialog.xml");
		filter.addExList("Menu.xml");
		filter.addExList("sound_setting.xml");
		filter.addExList("sound_setting_default.xml");
		filter.setDescription("XML File(*.xml)");
		chooser.setFileFilter(filter);

		setSettingFileChooser(chooser);
	}

	public void start() {
		try {
			setFontManager(new CGFFontManager(this));
			setFactory(new CGFFactory(this));
			setMainFrame(getFactory().createMainFrame());
			setGameData(getFactory().createGameData());
			getGameData().setMapSize(new CGFSize(20, 20));
			setDialogDocument(getFactory().createDialogDocumnet(
					getGameData().createHandlerMap()));

			getGameData().setDisplayMode();
			getMainFrame().autoFitSize();
			getMainFrame().activateMouseListner();

		} catch (Exception ex) {
			ex.printStackTrace();
			JOptionPane.showMessageDialog(null, "エラーが発生しました（" + ex + ")");
		}
	}

	public void finish() {
		System.exit(0);
	}

	public CGFMainFrame getMainFrame() {
		return mainFrame;
	}

	public void setMainFrame(CGFMainFrame mainFrame) {
		this.mainFrame = mainFrame;
	}

	public CGFGameData getGameData() {
		return gameData;
	}

	public void setGameData(CGFGameData gameData) {
		this.gameData = gameData;
	}

	public CGFFactory getFactory() {
		return factory;
	}

	public void setFactory(CGFFactory factory) {
		this.factory = factory;
	}

	public CGFDialogDocument getDialogDocument() {
		return dialogDocument;
	}

	public void setDialogDocument(CGFDialogDocument dialogDocument) {
		this.dialogDocument = dialogDocument;
	}

	public CGFMenuDocument getMenuDocument() {
		return menuDocument;
	}

	public void setMenuDocument(CGFMenuDocument menuDocument) {
		this.menuDocument = menuDocument;
	}

	public JFileChooser getSaveFileChooser() {
		return saveFileChooser;
	}

	public void setSaveFileChooser(JFileChooser saveFileChooser) {
		this.saveFileChooser = saveFileChooser;
	}

	public void setFontManager(CGFFontManager fontManager) {
		this.fontManager = fontManager;
	}

	public CGFFontManager getFontManager() {
		return fontManager;
	}

	public void setSetting(CGFSetting setting) {
		this.setting = setting;
	}

	public CGFSetting getSetting() {
		return setting;
	}

	public void setSoundSetting(CGFSoundSetting soundSetting) {
		getSoundManager().initDevice(soundSetting);
	}

	public CGFSoundSetting getSoundSetting() {
		return getSoundManager().getSoundSetting();
	}

	public void setSoundManager(CGFSoundManager soundManager) {
		this.soundManager = soundManager;
	}

	public CGFSoundManager getSoundManager() {
		return soundManager;
	}

	public void loadSoundLib() {
		String jllib = getSetting().getStringValue("jllib");
		try {
			addClassPath(jllib);
		} catch (IOException e) {
			e.printStackTrace();
		}
		setSoundManager(new CGFSoundManager());
	}

	public static void addClassPath(String s) throws IOException {
		File f = new File(s);
		addClassPath(f);
	}

	public static void addClassPath(File f) throws IOException {
		URL u = f.toURI().toURL();
		addClassPath(u);
	}

	public static void addClassPath(URL u) throws IOException {
		final Class[] parameters = new Class[] { URL.class };

		URLClassLoader sysloader = (URLClassLoader) ClassLoader
				.getSystemClassLoader();

		Class sysclass = URLClassLoader.class;

		try {
			Method method = sysclass.getDeclaredMethod("addURL", parameters);
			method.setAccessible(true);
			method.invoke(sysloader, new Object[] { u });
		} catch (Throwable t) {
			throw new IOException("ライブラリーが見つかりません　" + u);
		}
	}

	public void setSoundFileChooser(JFileChooser soundFileChooser) {
		this.soundFileChooser = soundFileChooser;
	}

	public JFileChooser getSoundFileChooser() {
		return soundFileChooser;
	}

	public void setSettingFileChooser(JFileChooser settingFileChooser) {
		this.settingFileChooser = settingFileChooser;
	}

	public JFileChooser getSettingFileChooser() {
		return settingFileChooser;
	}
	
}
