package sjima.cgf;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Transmitter;

import javazoom.jl.decoder.JavaLayerException;
import javazoom.jl.player.Player;

public class CGFSoundManager {
	private static final int MP3_BUFFER_SIZE = 102400;
	private MidiPlayerThread midiPlayerThread = new MidiPlayerThread();
	private Mp3PlayerThread mp3PlayerThread = new Mp3PlayerThread(true);
	private String currentSoundFile = null;
	private CGFSoundSetting soundSetting;
	private MidiDevice midiout = null;

	public CGFSoundManager() {
	}

	public void initDevice(CGFSoundSetting setting) {
		setSoundSetting(setting);
		if (setting.isDebug()) {
			dumpDevice();
		}
		MidiDevice.Info[] info = MidiSystem.getMidiDeviceInfo();
		for (int i = 0; i < info.length; i++) {
			try {
				MidiDevice dev = MidiSystem.getMidiDevice(info[i]);
				if (setting.getEnableMidioutSet().contains(info[i].toString())) {
					setMidiout(dev);
					if (setting.isDebug()) {
						System.out.println("MIDI out device - "
								+ info[i].toString());
					}
					break;
				}
			} catch (MidiUnavailableException e) {
				e.printStackTrace();
			}
		}
		setSoundSetting(setting);
	}

	public void dumpDevice() {
		MidiDevice.Info[] info = MidiSystem.getMidiDeviceInfo();
		for (int i = 0; i < info.length; i++) {
			try {
				MidiDevice dev = MidiSystem.getMidiDevice(info[i]);
				System.out.println("midi output : info[" + i + "] = "
						+ info[i].toString() + " - " + info[i].getDescription()
						+ "(" + dev.getClass().getName() + ")");
			} catch (MidiUnavailableException e) {
				e.printStackTrace();
			}
		}
	}

	public synchronized void playSound(String soundFile) {
		stopMp3();
		stopMidi();
		if (!getSoundSetting().isMusicEnableAndNoMute()) {
			return;
		}
		if (soundFile == null) {
			return;
		}
		if (soundFile.matches(".+[.][mM][pP]3")) {
			playMp3(soundFile);
		} else if (soundFile.matches(".+[.][mM][iI][dD]")) {
			playMidi(soundFile);
		}
	}

	public synchronized void playEffect(String soundFile) {
		if (!getSoundSetting().isEffectEnableAndNoMute()) {
			return;
		}

		if (soundFile.matches(".+[.][mM][pP]3")) {
			playMp3Effect(soundFile);
		} else if (soundFile.matches(".+[.][mM][iI][dD]")) {
			playMidiEffect(soundFile);
		}
	}

	private void playMidiEffect(String soundFile) {
		try {
			Sequence seq = MidiSystem.getSequence(new File(soundFile));
			Sequencer seqer = createSequencer(seq);
			startMidiSequencer(seqer);
		} catch (InvalidMidiDataException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void playMp3Effect(String soundFile) {
		CGFPlayer player = createMp3Player(soundFile);
		startMp3Player(player, false);
	}

	public synchronized void playMp3(String mp3File) {
		getMp3PlayerThread().startPlayer(mp3File);
		setCurrentSoundFile(mp3File);
	}

	public synchronized void stopMp3() {
		try {
			getMp3PlayerThread().stopPlayer();
		} catch (Exception e) {
		}
		setCurrentSoundFile(null);
	}

	public synchronized void playMidi(String midiFile) {
		getMidiPlayerThread().startPlayer(midiFile);
		setCurrentSoundFile(midiFile);
	}

	public synchronized void stopMidi() {
		try {
			getMidiPlayerThread().stopPlayer();
		} catch (Exception e) {
		}
		setCurrentSoundFile(null);
	}

	public void stopMidiSequencer(Sequencer seqer) {

		if (seqer == null) {
			return;
		}
		try {
			seqer.close();
		} catch (Exception e) {
		}

	}

	public void startMidiSequencer(Sequencer seqer) {

		if (seqer == null) {
			return;
		}
		try {
			if (!seqer.isOpen()) {
				seqer.open();
			}
			seqer.start();
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		}

	}

	public void stopMp3Player(CGFPlayer player) {
		player.close();
	}

	public void startMp3Player(CGFPlayer player, boolean isLoop) {
		(new Mp3StartThread(player, isLoop)).start();
	}

	public Mp3PlayerThread getMp3PlayerThread() {
		return mp3PlayerThread;
	}

	public void setCurrentSoundFile(String currentSoundFile) {
		this.currentSoundFile = currentSoundFile;
	}

	public String getCurrentSoundFile() {
		return currentSoundFile;
	}

	public CGFPlayer createMp3Player(String mp3File) {
		try {
			InputStream stream = new BufferedInputStream((new FileInputStream(
					mp3File)), MP3_BUFFER_SIZE);
			return new CGFPlayer(stream);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (JavaLayerException e) {
			e.printStackTrace();
		}
		return null;
	}

	public Sequencer createSequencer(Sequence seq) {
		if (seq == null) {
			return null;
		}
		try {
			Sequencer seqer = MidiSystem.getSequencer();
			seqer.setSequence(seq);
			if (getMidiout() != null) {
				try {
					if (!getMidiout().isOpen()) {
						getMidiout().open();
					}
					for (Iterator ite = seqer.getTransmitters().iterator(); ite
							.hasNext();) {
						Transmitter trans = (Transmitter) ite.next();
						trans.close();
					}
					seqer.getTransmitter().setReceiver(
							getMidiout().getReceiver());
				} catch (Exception ex) {
					ex.printStackTrace(); // Vista対策:強行!
					setMidiout(null);
				}
			}

			return seqer;
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		} catch (InvalidMidiDataException e) {
			e.printStackTrace();
		}
		return null;
	}

	public void setSoundSetting(CGFSoundSetting soundSetting) {
		this.soundSetting = soundSetting;
	}

	public CGFSoundSetting getSoundSetting() {
		return soundSetting;
	}

	public MidiPlayerThread getMidiPlayerThread() {
		return midiPlayerThread;
	}

	public class MidiPlayerThread extends Thread {
		private static final long SLEEP_MSEC = 250;
		private Sequencer sequencer = null;
		private String midiFile = null;
		private boolean isRunning = false;

		public MidiPlayerThread() {
			// 非力な環境ではMIDIはタイミングが狂いやすいので、優先度最大
			this.setPriority(MAX_PRIORITY);
		}

		public void run() {
			setRunning(true);
			String currentFile = null;
			Sequence currentSeq = null;
			while (true) {
				try {
					if (getMidiFile() == null) {
						if (currentFile == null) {
							Thread.sleep(SLEEP_MSEC);
							continue;
						} else {
							stopMidiSequencer(getSequencer());
							setSequencer(null);
							currentFile = null;
							currentSeq = null;
						}
					} else {
						if (currentFile != null) {
							if (getMidiFile().equals(currentFile)) {
								if (getSequencer().isRunning()) {
									Thread.sleep(SLEEP_MSEC);
									continue;
								}
							}
							stopMidiSequencer(getSequencer());
						}
						currentFile = getMidiFile();
						currentSeq = MidiSystem.getSequence(new File(
								currentFile));
						setSequencer(createSequencer(currentSeq));
						startMidiSequencer(getSequencer());
					}
				} catch (InterruptedException e) {
				} catch (InvalidMidiDataException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		public void stopPlayer() {
			setMidiFile(null);
			if (!isRunning()) {
				this.start();
			}
		}

		public void startPlayer(String midiFile) {
			setMidiFile(midiFile);
		}

		public void setSequencer(Sequencer sequencer) {
			this.sequencer = sequencer;
		}

		public Sequencer getSequencer() {
			return sequencer;
		}

		public void setMidiFile(String midiFile) {
			this.midiFile = midiFile;
		}

		public String getMidiFile() {
			return midiFile;
		}

		public void setRunning(boolean isRunning) {
			this.isRunning = isRunning;
		}

		public boolean isRunning() {
			return isRunning;
		}

	}

	public class Mp3PlayerThread extends Thread {
		private CGFPlayer player = null;
		private String mp3File = null;
		private boolean isRunning = false;
		private boolean isLoop = false;

		public Mp3PlayerThread(boolean isLoop) {
			this.setPriority(MIN_PRIORITY);
			setLoop(isLoop);
		}

		public void run() {
			setRunning(true);
			String currentFile = null;
			while (true) {
				try {
					if (getMp3File() == null) {
						if (currentFile == null) {
							Thread.sleep(100);
							continue;
						} else {
							stopMp3Player(getPlayer());
							setPlayer(null);
							currentFile = null;
						}
					} else {
						if (currentFile != null) {
							if (getMp3File().equals(currentFile)) {
								if (getPlayer().isInPlay()) {
									Thread.sleep(100);
									continue;
								} else if (!isLoop()) {
									break;
								}
							}
							stopMp3Player(getPlayer());
						}
						currentFile = getMp3File();
						CGFPlayer wPlayer = createMp3Player(currentFile);
						startMp3Player(wPlayer, isLoop());
						setPlayer(wPlayer);
						Thread.sleep(100);
					}
				} catch (InterruptedException e) {
				}
			}
		}

		public void stopPlayer() {
			setMp3File(null);
			if (!isRunning()) {
				this.start();
			}
		}

		public void startPlayer(String mp3File) {
			setMp3File(mp3File);
		}

		public void setRunning(boolean isRunning) {
			this.isRunning = isRunning;
		}

		public boolean isRunning() {
			return isRunning;
		}

		public void setPlayer(CGFPlayer player) {
			this.player = player;
		}

		public CGFPlayer getPlayer() {
			return player;
		}

		public void setMp3File(String mp3File) {
			this.mp3File = mp3File;
		}

		public String getMp3File() {
			return mp3File;
		}

		public void setLoop(boolean isLoop) {
			this.isLoop = isLoop;
		}

		public boolean isLoop() {
			return isLoop;
		}

	}

	public static class Mp3StartThread extends Thread {
		private CGFPlayer player = null;

		public Mp3StartThread(CGFPlayer player, boolean isLoop) {
			if (isLoop) {
				this.setPriority(Thread.MAX_PRIORITY);
			} else {
				this.setPriority(Thread.MIN_PRIORITY);
			}
			this.player = player;
		}

		public void run() {
			try {
				player.play();
				player = null;
			} catch (JavaLayerException e) {
				e.printStackTrace();
			}
		}
	}

	public void stopSound() {
		stopMp3();
		stopMidi();
	}

	private void setMidiout(MidiDevice midiout) {
		this.midiout = midiout;
	}

	private MidiDevice getMidiout() {
		return midiout;
	}
	
	public static class CGFPlayer {
		private boolean inPlay = false;
		private Player player;
		private InputStream stream = null;

		public CGFPlayer(InputStream stream) throws JavaLayerException {
			setStream(stream);
			setPlayer(new Player(stream));
		}
		
		private void setStream(InputStream stream2) {
			stream = stream2;
		}
		
		private InputStream getStream() {
			return stream;
		}

		public void close() {
			if (getPlayer() == null) {
				return;
			}
			if (isInPlay()) {
				setInPlay(false);
				setPlayer(null);
				try {
					getStream().close();
				} catch (IOException e) {
				}
				setStream(null);
			} else {
				setPlayer(null);
			}
		}

		public void play() throws JavaLayerException {
			if (getPlayer() == null) {
				return;
			}
			try {
				setInPlay(true);
				getPlayer().play();
			} catch (Exception e) {
			} finally {
				setInPlay(false);
			}
		}

		public void setInPlay(boolean inPlay) {
			this.inPlay = inPlay;
		}

		public boolean isInPlay() {
			return inPlay;
		}

		public void setPlayer(Player player) {
			this.player = player;
		}

		public Player getPlayer() {
			return player;
		}
		
	}

}
