/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf.layer;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

import sjima.cgf.CGFDialog;
import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFLayer;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFMenuSet;
import sjima.cgf.CGFSize;
import sjima.cgf.CGFUtil;

public class CGFDialogLayer extends CGFLayer {

	private CGFDialog currentDispDialog;
	private CGFMenuSet currentDispMenu;
	private final static Color CLEAR_COLOR = new Color(255, 255, 255, 0);
	public final static String LAYER_NAME = "dialogLayer";
	private Rectangle lastDialogRectangle = null;
	private Rectangle lastMenuRectangle = null;
	private BufferedImage dialogImage = null;
	private BufferedImage menuImage = null;

	public CGFDialogLayer(CGFMain main, CGFSize size) {
		super(main, size);
	}

	public String getLayerName() {
		return LAYER_NAME;
	}

	public List displayDialog(CGFDialogAction action) {
		List result = clearDialog();
		result.addAll(clearMenu());
		CGFDialog dialog = getMain().getDialogDocument().getDialog(
				action.getDialogName());
		if (dialog != null) {
			setCurrentDispDialog(dialog);
			setDialogImage(CGFUtil.createImage(new CGFSize(
					dialog.getSize().width, dialog.getSize().height)));
			dialog.setEnable(true);
			action.setup();
			dialog.updateImage();
			result.addAll(drawDialog());
		}

		return result;
	}

	public List displayMenu(CGFDialogAction action) {
		List result = clearMenu();
		result.addAll(clearDialog());
		CGFMenuSet menuSet = getMain().getMenuDocument().getMenuSet(
				action.getDialogName());
		if (menuSet != null) {
			setCurrentDispMenu(menuSet);
			setMenuImage(CGFUtil.createImage(new CGFSize(
					menuSet.getSize().width, menuSet.getSize().height)));
			menuSet.setEnable(true);
			action.setup();
			menuSet.updateImage();
			result.addAll(drawMenu());
		}

		return result;
	}

	public List redrawDialog() {
		if (getCurrentDispDialog() == null) {
			return new ArrayList();
		}
		getCurrentDispDialog().updateImage();
		List list = clearDialog();
		list.addAll(drawDialog());
		return list;
	}

	public List redrawMenu() {
		getCurrentDispMenu().updateImage();
		List list = clearMenu();
		list.addAll(drawMenu());
		return list;
	}

	public List drawDialog() {
		List result = new ArrayList();

		if (getCurrentDispDialog() == null) {
			return result;
		}

		Graphics2D gc = getDialogImage().createGraphics();
		Rectangle rect = getDialogRectangle();
		getCurrentDispDialog().setPoint(new Point(rect.x, rect.y));
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, rect.width, rect.height);
		gc.drawImage(getCurrentDispDialog().getImage(), 0, 0, null);
		result.add(rect);
		setLastDialogRectangle(rect);

		return result;
	}

	public List drawMenu() {
		List result = new ArrayList();

		if (getCurrentDispMenu() == null) {
			return result;
		}

		Graphics2D gc = getMenuImage().createGraphics();
		Rectangle rect = getMenuRectangle();
		getCurrentDispMenu().setPoint(new Point(rect.x, rect.y));
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, rect.width, rect.height);
		gc.drawImage(getCurrentDispMenu().getImage(), 0, 0, null);
		result.add(rect);
		setLastMenuRectangle(rect);

		return result;
	}

	public List clearDialog() {
		List result = new ArrayList();
		if (getLastDialogRectangle() == null) {
			return result;
		}

		Graphics2D gc = getDialogImage().createGraphics();
		Rectangle rect = getLastDialogRectangle();
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, rect.width, rect.height);
		result.add(rect);

		return result;
	}

	public List clearMenu() {
		List result = new ArrayList();

		Rectangle rect = getLastMenuRectangle();
		if (rect != null) {
			Graphics2D gc = getMenuImage().createGraphics();
			gc.setBackground(CLEAR_COLOR);
			gc.clearRect(0, 0, rect.width, rect.height);
			result.add(rect);
		}

		return result;
	}

	public Rectangle getDialogRectangle() {
		Rectangle result = new Rectangle();

		Rectangle viewPoint = getMain().getMainFrame().getViewPort();
		viewPoint = viewPoint.intersection(getMain().getGameData().getBoardRectangle());
		result.width = getCurrentDispDialog().getSize().width;
		result.height = getCurrentDispDialog().getSize().height;
		result.x = viewPoint.x + (viewPoint.width - result.width) / 2;
		result.y = viewPoint.y + (viewPoint.height - result.height) / 2;
		if (result.x < 0) {
			result.x = 0;
		}
		if (result.y < 0) {
			result.y = 0;
		}

		return result;
	}

	public Rectangle getMenuRectangle() {
		Rectangle result = new Rectangle();
		CGFLocation loc = getMain().getGameData().getCursolLocation();
		Point point = getMain().getGameData().getLocationPoint(loc);

		Rectangle viewPoint = getMain().getMainFrame().getViewPort();
		result.width = getCurrentDispMenu().getSize().width;
		result.height = getCurrentDispMenu().getSize().height;
		result.x = point.x + getMain().getGameData().getSquareSize().width;
		result.y = point.y + getMain().getGameData().getSquareSize().height;
		if ((viewPoint.x + viewPoint.width) < (result.x + result.width)) {
			result.x = point.x - result.width;
		}
		if ((viewPoint.y + viewPoint.height) < (result.y + result.height)) {
			result.y = viewPoint.y + viewPoint.height - result.height;
		}

		return result;
	}

	public CGFDialog getCurrentDispDialog() {
		return currentDispDialog;
	}

	public void setCurrentDispDialog(CGFDialog currentDispDialog) {
		this.currentDispDialog = currentDispDialog;
	}

	public boolean hasCurrentDispDialog() {
		return (getCurrentDispDialog() != null);
	}

	public Rectangle getLastDialogRectangle() {
		return lastDialogRectangle;
	}

	public void setLastDialogRectangle(Rectangle lastDialogRectangle) {
		this.lastDialogRectangle = lastDialogRectangle;
	}

	public Rectangle getLastMenuRectangle() {
		return lastMenuRectangle;
	}

	public void setLastMenuRectangle(Rectangle lastMenuRectangle) {
		this.lastMenuRectangle = lastMenuRectangle;
	}

	public CGFMenuSet getCurrentDispMenu() {
		return currentDispMenu;
	}

	public void setCurrentDispMenu(CGFMenuSet currentDispMenu) {
		this.currentDispMenu = currentDispMenu;
	}

	public boolean hasCurrentDispMenu() {
		return (getCurrentDispMenu() != null);
	}

	public void setDialogImage(BufferedImage dialogImage) {
		this.dialogImage = dialogImage;
	}

	public BufferedImage getDialogImage() {
		return dialogImage;
	}

	public void setMenuImage(BufferedImage menuImage) {
		this.menuImage = menuImage;
	}

	public BufferedImage getMenuImage() {
		return menuImage;
	}

	public void clearAll() {
		clearDialog();
		clearMenu();
	}

	public void paint(BufferedImage boardImage) {
		if (getLastDialogRectangle() != null) {
			paintRectangle(boardImage, getLastDialogRectangle());
		}
		if (getLastMenuRectangle() != null) {
			paintRectangle(boardImage, getLastMenuRectangle());
		}
	}

	public void paintRectangle(BufferedImage boardImage, Rectangle rect) {
		if (!isVisible()) {
			return;
		}
		paintMenu(boardImage, rect);
		paintDialog(boardImage, rect);
	}

	public void paintDialog(BufferedImage boardImage, Rectangle rect) {
		Rectangle dialogRect = getLastDialogRectangle();
		if (dialogRect == null) {
			return;
		}
		if (!dialogRect.intersects(rect)) {
			return;
		}
		Rectangle drawRect = dialogRect.intersection(rect);
		Rectangle inRect = CGFUtil.getInnerRectangle(dialogRect, drawRect);
		Graphics2D gc = boardImage.createGraphics();
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
				getTransparency());
		gc.setComposite(ac);
		gc.drawImage(getDialogImage(), drawRect.x, drawRect.y, drawRect.x
				+ drawRect.width, drawRect.y + drawRect.height, inRect.x,
				inRect.y, inRect.width + inRect.x, inRect.height + inRect.y,
				null);
	}

	public void paintMenu(BufferedImage boardImage, Rectangle rect) {
		Rectangle menuRect = getLastMenuRectangle();
		if (menuRect == null) {
			return;
		}
		if (!menuRect.intersects(rect)) {
			return;
		}
		Rectangle drawRect = menuRect.intersection(rect);
		Rectangle inRect = CGFUtil.getInnerRectangle(menuRect, drawRect);
		Graphics2D gc = boardImage.createGraphics();
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
				getTransparency());
		gc.setComposite(ac);
		gc.drawImage(getMenuImage(), drawRect.x, drawRect.y, drawRect.x
				+ drawRect.width, drawRect.y + drawRect.height, inRect.x,
				inRect.y, inRect.width + inRect.x, inRect.height + inRect.y,
				null);
	}
}
