package sjima.wop;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.swing.JOptionPane;

import org.w3c.dom.Document;

import sjima.cgf.CGFFontManager;
import sjima.cgf.CGFGameData;
import sjima.cgf.CGFLayer;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFMode;
import sjima.cgf.CGFRowColsTableDataBuilder;
import sjima.cgf.CGFSize;
import sjima.cgf.CGFXMLUtil;
import sjima.cgf.layer.CGFCursolLayer;
import sjima.cgf.layer.CGFDialogLayer;
import sjima.cgf.layer.CGFInfoBoardLayer;
import sjima.wop.action.WOPMessageDialogAction;
import sjima.wop.event.WOPEventManager;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.layer.WOPTerrainLayer;
import sjima.wop.mode.WOPDialogMode;
import sjima.wop.mode.WOPDisplayMode;
import sjima.wop.mode.WOPImageSaveMode;

public class WOPGameData extends CGFGameData {
	public static final File SAVE_DIR = new File(System
			.getProperty("user.home"), "WOPSaveGame");
	public static final File USER_DIR = new File(System.getProperty("user.dir"));
	private final static Color CLEAR_COLOR = new Color(255, 255, 255, 0);

	private LinkedHashMap modeList = new LinkedHashMap();
	private List messageList = new ArrayList();
	private WOPMapInfo mapInfo;
	private WOPMapDocument mapDocument;
	private WOPForceManager forceManager;
	private WOPEventManager eventManager;
	private Map aiMap = new HashMap();
	private List combatResultList = new ArrayList();
	private int endTurn;

	public WOPGameData(CGFMain p_main) {
		super(p_main);
		setSquareSize(new CGFSize(32, 32));
	}

	public void buildModes() {
		setModeList(new LinkedHashMap());
		CGFMode mode = new WOPDisplayMode(getMain());
		getModeList().put(mode.getModeName(), mode);
		mode = new WOPDialogMode(getMain());
		getModeList().put(mode.getModeName(), mode);
	}

	private void createAiMap() {
		aiMap.clear();
		for (Iterator ite = getPlayerList().iterator(); ite.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			WOPAI ai = new WOPAIStandard(getMain(), player);
			aiMap.put(player.getNo(), ai);
		}
	}

	public void buildLayers() {
		LinkedHashMap layerList = new LinkedHashMap();
		setLayerList(layerList);
		CGFLayer layer = new WOPTerrainLayer(getMain(), getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new WOPForceLayer(getMain(), getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new CGFCursolLayer(getMain(), getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new CGFInfoBoardLayer(getMain(), getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
		layer = new CGFDialogLayer(getMain(), getBoardImageSize());
		layerList.put(layer.getLayerName(), layer);
	}

	public void displayMessageList(List msg, Runnable handler) {
		displayMessageList(msg, handler, getMessageWaitTime());
	}

	public void displayMessageList(List msg, Runnable handler,
			boolean closeEnable) {
		displayMessageList(msg, handler, getMessageWaitTime(), closeEnable);
	}

	public void displayMessageList(List msg, Runnable handler, int timeout) {
		displayMessageList(msg, handler, timeout, true);
	}

	public void displayMessageList(List msg, Runnable handler, int timeout,
			boolean closeEnable) {
		WOPDialogMode mode = (WOPDialogMode) this.getModeList().get(
				WOPDialogMode.MODE_NAME);
		WOPMessageDialogAction action = (WOPMessageDialogAction) mode
				.getActionMap().get(WOPMessageDialogAction.DIALOG_NAME);
		action.setTimeout(timeout);
		action.setCloseEnable(closeEnable);
		getMessageList().clear();
		getMessageList().addAll(msg);
		displayDialog(WOPMessageDialogAction.DIALOG_NAME, handler);
	}

	public void displayDialog(String displayName) {
		displayDialog(displayName, null);
	}

	public synchronized void displayDialog(String displayName, Runnable handler) {
		changeCurrentMode(WOPDialogMode.MODE_NAME);
		WOPDialogMode dialogMode = (WOPDialogMode) getCurrentMode();
		dialogMode.setFinishHandler(handler);
		dialogMode.dispDialog(dialogMode.getAction(displayName));
	}

	public List getMessageList() {
		return messageList;
	}

	public int getMessageWaitTime() {
		if (isAllAi()) {
			return getMain().getSetting()
					.getIntegerValue("aionly_message_wait");
		} else {
			return getMain().getSetting().getIntegerValue("message_wait");
		}
	}

	public boolean isAllAi() {
		for (int i = 0; i < getNinzu(); i++) {
			WOPPlayer player = (WOPPlayer) getPlayerList().get(i);
			if (!"1".equals(player.getAi())) {
				return false;
			}
		}
		return true;
	}

	public LinkedHashMap getModeList() {
		return modeList;
	}

	public void setModeList(LinkedHashMap modeList) {
		this.modeList = modeList;
	}

	public void loadGameData(String path) throws Exception {
		Document doc = CGFXMLUtil.loadXML(path);
		setMapDocument((WOPMapDocument) getMain().getFactory()
				.createMapDocument(doc));
		setTerrainMap(getMapDocument().getTerrainMap());
		setMapSize(getMapDocument().getMapSize());
		if ((!"".equals(getMapDocument().getMaxPlayer()))
				&& (getMapDocument().getMaxPlayer() != null)) {
			setMaxPlayer(Integer.parseInt(getMapDocument().getMaxPlayer()));
		}
		if ((!"".equals(getMapDocument().getEndTurn()))
				&& (getMapDocument().getMaxPlayer() != null)) {
			setEndTurn(Integer.parseInt(getMapDocument().getEndTurn()));
		}
		CGFRowColsTableDataBuilder builder = getMain().getFactory()
				.getCellBuilder();
		setCellTable(getMapDocument().getCellTable(builder));
		setColorList(getMapDocument().getColorList());
		setPlayerList(getMapDocument().getPlayerList());
		createPlayerMap();
		createAiMap();
		setNinzu(getMapDocument().getPlayerCount());
		setForceManager(getMapDocument().getForceManager());
		setCombatResultList(getMapDocument().getCombatResultList());
		setEventManager(getMapDocument().getEventManager());
		setMapInfo(new WOPMapInfo(getMain(), path));
	}

	public void createDefaultTerrain() {
		WOPTerrain terrain;
		if (getMapDocument() != null) {
			terrain = (WOPTerrain) getMapDocument().getDefaultTerrain(
					getTerrainMap());
			setDefaultTerrain(terrain);
		}

	}

	private void setMapDocument(WOPMapDocument mapDocument) {
		this.mapDocument = mapDocument;
	}

	public WOPMapDocument getMapDocument() {
		return this.mapDocument;
	}

	public void setMapInfo(WOPMapInfo mapInfo) {
		this.mapInfo = mapInfo;
	}

	public WOPMapInfo getMapInfo() {
		return this.mapInfo;
	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public WOPAI getAI(String no) {
		return (WOPAI) getAiMap().get(no);
	}

	public void setForceManager(WOPForceManager forceManager) {
		this.forceManager = forceManager;
	}

	public WOPForceManager getForceManager() {
		return forceManager;
	}

	public WOPPlayer getPlayer(String no) {
		return (WOPPlayer) super.getPlayer(no);
	}

	public List getAllCellList() {
		return getPlayersCellList(null);
	}

	public List getPlayersCellList(String pno) {
		List result = new ArrayList();
		CGFLocation loc = new CGFLocation(0, 0);
		for (loc.row = 0; loc.row < getMapSize().height; loc.row++) {
			for (loc.col = 0; loc.col < getMapSize().width; loc.col++) {
				WOPCell cell = (WOPCell) getCell(loc);
				if (pno == null) {
					result.add(cell);
					continue;
				}
				String ctrl = getForceManager().getControll(loc);
				if (pno.equals(ctrl)) {
					result.add(cell);
				}
			}
		}
		return result;
	}

	public List createAjustList(CGFLocation startLoc) {
		List ajustList = new ArrayList();
		List aList = startLoc.getAjustList();
		for (Iterator ite = aList.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			if (getCell(loc) != null) {
				ajustList.add(loc);
			}
		}

		return ajustList;
	}

	public List createAjustCellList(CGFLocation startLoc) {
		List ajustList = new ArrayList();
		List aList = startLoc.getAjustList();
		for (Iterator ite = aList.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			WOPCell cell = getCell(loc);
			if (cell != null) {
				ajustList.add(cell);
			}
		}

		return ajustList;
	}

	public WOPCell getCell(CGFLocation loc) {
		return (WOPCell) super.getCell(loc);
	}

	public Map getAiMap() {
		return aiMap;
	}

	public void setAiMap(Map aiMap) {
		this.aiMap = aiMap;
	}

	public List getAllTargetLocList() {
		List result = new ArrayList();
		for (Iterator ite = getPlayerList().iterator(); ite.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			CGFLocation loc = player.getTargetLoc();
			if (loc != null) {
				result.add(loc);
			}
		}
		return result;
	}

	public List getPlayerNoAtTargetLoc(CGFLocation loc) {
		List playerNoList = new ArrayList();
		for (Iterator ite = getPlayerList().iterator(); ite.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			if (loc.equals(player.getTargetLoc())) {
				playerNoList.add(player.getNo());
			}
		}
		return playerNoList;
	}

	public void calTotalFertility() {
		for (Iterator ite = getPlayerList().iterator(); ite.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			int totalFertility = 0;
			List list = getPlayersCellList(player.getNo());
			for (Iterator ite2 = list.iterator(); ite2.hasNext();) {
				WOPCell cell = (WOPCell) ite2.next();
				totalFertility += cell.getTerrain().getFertility(player);
			}
			player.setTotalFertility(totalFertility);
		}
	}

	// 戦闘結果を格納する
	public void storeCombatResult(int turn, WOPCell cell, String defender,
			String attacker, int defenderStrength, int attackerStrength,
			int defenderForce, int attackerForce, int defenderDamage,
			int attackerDamage, int result, WOPCell retreat) {
		WOPCombatResult cr = new WOPCombatResult(getMain(), turn, cell,
				WOPCombatResult.TYPE_BATTLE, defender, attacker,
				defenderStrength, attackerStrength, defenderForce,
				attackerForce, defenderDamage, attackerDamage, result, retreat);
		// cr.displayMessage();
		getCombatResultList().add(cr);
	}

	// 反乱結果を格納する
	public void storeRevoltResult(int turn, WOPCell cell, String defender,
			String attacker, int strength) {
		WOPCombatResult cr = new WOPCombatResult(getMain(), turn, cell,
				WOPCombatResult.TYPE_REVOLT, defender, attacker, strength,
				strength, strength, strength, strength, 0, 1, null);
		cr.displayMessage();
		getCombatResultList().add(cr);
	}

	public List getCombatResultList() {
		return combatResultList;
	}

	public void setCombatResultList(List combatResultList) {
		this.combatResultList = combatResultList;
	}

	public void clearAllTarget() {
		List repaintList = new ArrayList();
		List targetLocList = new ArrayList();
		targetLocList.addAll(getMain().getGameData().getAllTargetLocList());

		for (Iterator ite = getPlayerList().iterator(); ite.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			player.setTargetLoc(null);
		}

		for (Iterator ite = targetLocList.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			WOPForceLayer layer = (WOPForceLayer) getMain().getGameData()
					.getLayer(WOPForceLayer.LAYER_NAME);
			repaintList.add(layer.repaintAtLocation(loc));
		}

		repaintRectangleList(repaintList);
	}

	public CGFLocation calBarycenter(String pno) {
		long row = 0;
		long col = 0;
		int totalStrength = 0;
		List forceLocList = getMain().getGameData().getPlayersCellList(pno);
		for (Iterator ite = forceLocList.iterator(); ite.hasNext();) {
			WOPCell cell = (WOPCell) ite.next();
			CGFLocation loc = cell.getLocation();
			int strength = getMain().getGameData().getForceManager()
					.getStrength(loc, pno);
			row += loc.row * strength;
			col += loc.col * strength;
			totalStrength += strength;
		}
		if (totalStrength == 0) {
			return new CGFLocation(
					getMain().getGameData().getMapSize().height / 2, getMain()
							.getGameData().getMapSize().width / 2);
		}
		CGFLocation result = new CGFLocation();
		result.row = (int) ((double) row / totalStrength + 0.5001);
		result.col = (int) ((double) col / totalStrength + 0.5001);

		return result;
	}

	public void setEventManager(WOPEventManager eventManager) {
		this.eventManager = eventManager;
	}

	public WOPEventManager getEventManager() {
		return eventManager;
	}

	public List getExtinctionList() {
		List result = new ArrayList();

		for (Iterator ite = getPlayerList().iterator(); ite.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			if (player.getTotalStreangth() == 0) {
				result.add(player);
			}
		}

		return result;
	}

	public void dispEventArea(CGFLocation targetLoc, Collection targetList,
			Color color) {
		List repaintList = new ArrayList();
		WOPTerrainLayer layer = (WOPTerrainLayer) getLayer(WOPTerrainLayer.LAYER_NAME);
		for (Iterator ite = targetList.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			repaintList.add(layer.eventAtLocation(loc, color));
		}
		getMain().getGameData().repaintRectangleList(repaintList);
		scrollToLocation(targetLoc);
	}

	public void setEndTurn(int endTurn) {
		this.endTurn = endTurn;
	}

	public int getEndTurn() {
		return endTurn;
	}

	public List getCellListByTtypeSet(Set ttypeSet) {
		List result = new ArrayList();
		for (Iterator ite = getAllCellList().iterator(); ite.hasNext();) {
			WOPCell cell = (WOPCell) ite.next();
			String ttype = cell.getTerrain().getTerrain();
			if (ttypeSet.contains(ttype)) {
				result.add(cell);
			}
		}

		return result;
	}

	static public BufferedImage createMapImage(String path) {
		Document doc;
		try {
			doc = CGFXMLUtil.loadXML(path);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		WOPMain main = new WOPMain();
		main.setFontManager(new CGFFontManager(main));
		main.setFactory(new WOPFactory(main));
		WOPGameData gameData = (WOPGameData) main.getFactory().createGameData();
		main.setGameData(gameData);

		gameData.setMapDocument((WOPMapDocument) main.getFactory()
				.createMapDocument(doc));
		gameData.setMapSize(gameData.getMapDocument().getMapSize());
		gameData.setTerrainMap(gameData.getMapDocument().getTerrainMap());
		gameData.setBorderTerrainMap(gameData.getMapDocument()
				.getBorderTerrainMap());
		CGFRowColsTableDataBuilder builder = main.getFactory().getCellBuilder();
		gameData.setCellTable(gameData.getMapDocument().getCellTable(builder));
		gameData.getMapDocument().setCellSide(gameData.getCellTable());

		gameData.setColorList(gameData.getMapDocument().getColorList());
		gameData.setPlayerList(gameData.getMapDocument().getPlayerList());
		gameData.createPlayerMap();
		gameData.createAiMap();
		gameData.setNinzu(gameData.getMapDocument().getPlayerCount());

		gameData.setForceManager(gameData.getMapDocument().getForceManager());
		BufferedImage image = new BufferedImage(gameData.getBoardImage()
				.getWidth(), gameData.getBoardImage().getHeight(), gameData
				.getBoardImage().getType());
		Graphics2D gc = gameData.getBoardImage().createGraphics();
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, gameData.getBoardImage().getWidth(), gameData
				.getBoardImage().getHeight());
		for (Iterator ite = gameData.getLayerList().values().iterator(); ite
				.hasNext();) {
			CGFLayer layer = (CGFLayer) ite.next();
			if (WOPTerrainLayer.LAYER_NAME.equals(layer.getLayerName())) {
			} else if (WOPForceLayer.LAYER_NAME.equals(layer.getLayerName())) {
			} else {
				continue;

			}
			layer.update();
			layer.paint(image);
		}
		return image;
	}

	public void saveGameData(String path) {
		getMapDocument().saveGameData(path);
	}

	public void saveImage(String path) {
		try {
			CGFMode curMode = getCurrentMode();
			changeCurrentMode(WOPImageSaveMode.MODE_NAME);

			ImageIO.write(getBoardImage(), "png", new File(path));

			changeCurrentMode(curMode.getModeName());

		} catch (IOException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "イメージファイルの保存に失敗しました");
		}
	}

	public void createSaveDir() {
		if (!SAVE_DIR.isDirectory()) {
			SAVE_DIR.mkdir();
		}
	}
	
	public WOPTurnManager getTurnManager() {
		return (WOPTurnManager)super.getTurnManager();
	}

	public Map getTerrainMap() {
		return super.getTerrainMap();
	}
}
